import Flutter
import UIKit

public class SwiftCoolLocationFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftCoolLocationFlutterPlugin()
        instance.initEventChannel(registrar: registrar)
        instance.initMethodChannel(registrar: registrar)
    }
    
    public func initEventChannel(registrar: FlutterPluginRegistrar){
        let eventChannel = FlutterEventChannel(name: "native_channel_id_456",
                                               binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler({
            func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
                let duration = arguments as? Int ?? 1_000
                CoolLocation.getLocationUpdates(duration.toLong()) {(latitude, longitude) in
                    events(["latitude" : latitude, "longitude" : longitude])
                }
                return nil
            }
            
            func onCancel(withArguments arguments: Any?) -> FlutterError? {
                CoolLocation.stopPeriodicUpdates()
                return nil
            }
        }())
    }
    
    public func initMethodChannel(registrar: FlutterPluginRegistrar){
        
        let channel = FlutterMethodChannel(name: "native_channel_id_123",
                                           binaryMessenger: registrar.messenger())
        channel.setMethodCallHandler({(call: FlutterMethodCall, result: FlutterResult) in
            switch call.method{
            case "getCurrentLocation":
                do{
                    let (latitude, longitude) = CoolLocation.getCurrentLocation()
                    result(["latitude" : latitude, "longitude" : longitude])
                } catch let error {
                    result(FlutterError(code: "1002", message: error.localizedDescription, details: nil))
                }
                
            default:
                result(FlutterError(code: "1001", message: "Unknown operation", details: nil))
            }
        })
    }
    
}
